library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.axi3intercon_pkg.all;
use work.axi_aw_router_pkg.all;
use work.axi_w_router_pkg.all;

entity axi3_intercon_w_router is
	port(
		clk            : in  std_logic;
		rst            : in  std_logic;
		masters_in     : out axi_w_masters_in_t(0 to MASTER_COUNT - 1);
		masters_out    : in  axi_w_masters_out_t(0 to MASTER_COUNT - 1);
		slaves_in      : out axi_w_slaves_in_t(0 to SLAVE_COUNT);
		slaves_out     : in  axi_w_slaves_out_t(0 to SLAVE_COUNT);
		write_locks    : in  write_locks_t;
		write_releases : out write_releases_t
	);
end entity axi3_intercon_w_router;

architecture RTL of axi3_intercon_w_router is
	type w_state_t is (W_READY, W_ACTIVE);
	type w_states_t is array (0 to MASTER_COUNT - 1) of w_state_t;
	signal w_states : w_states_t;
	signal wready_s : std_logic_vector(0 to MASTER_COUNT - 1);
begin
	wready_gen : for i in 0 to MASTER_COUNT - 1 generate
		masters_in(i).wready <= wready_s(i);
	end generate wready_gen;

	w_router_sync : process(clk, rst) is
	begin
		if rst = '1' then
			for i in 0 to MASTER_COUNT - 1 loop
				wready_s(i)       <= '0';
				write_releases(i) <= '0';
			end loop;
			for i in 0 to SLAVE_COUNT loop
				slaves_in(i).wvalid <= '0';
				slaves_in(i).wid    <= (others => '0');
				slaves_in(i).wlast  <= '0';
				slaves_in(i).wstrb  <= (others => '0');
				slaves_in(i).wuser  <= (others => '0');
				slaves_in(i).wdata  <= (others => '0');
			end loop;
		elsif rising_edge(clk) then
			for i in 0 to MASTER_COUNT - 1 loop
				write_releases(i) <= '0';
				if write_locks(i).locked = '1' then
					case w_states(i) is
						when W_READY =>
							wready_s(i) <= '1';
							if wready_s(i) = '1' and masters_out(i).wvalid = '1' then
								wready_s(i)                                <= '0';
								slaves_in(write_locks(i).slave_idx).wdata  <= masters_out(i).wdata;
								slaves_in(write_locks(i).slave_idx).wid    <= std_logic_vector(to_unsigned(i, WID_SLAVE_BITS - WID_MASTER_BITS)) & masters_out(i).wid;
								slaves_in(write_locks(i).slave_idx).wlast  <= masters_out(i).wlast;
								slaves_in(write_locks(i).slave_idx).wstrb  <= masters_out(i).wstrb;
								slaves_in(write_locks(i).slave_idx).wuser  <= masters_out(i).wuser;
								slaves_in(write_locks(i).slave_idx).wvalid <= '1';
								if masters_out(i).wlast = '1' then
									write_releases(i) <= '1';
								end if;
								w_states(i) <= W_ACTIVE;

							end if;
						when W_ACTIVE =>
							if slaves_out(write_locks(i).slave_idx).wready = '1' then
								slaves_in(write_locks(i).slave_idx).wvalid <= '0';
								w_states(i)                                <= W_READY;
							end if;
					end case;
				else
					slaves_in(i).wvalid <= '0';
					wready_s(i)         <= '0';
				end if;
			end loop;
		end if;
	end process w_router_sync;

end architecture RTL;
